@@ -53,8 +53,12 @@
 
 #define SCROLL_STEP_SIZE 32
 
+/* Maximum zoom factor */
+
+#define MAX_ZOOM_FACTOR 128
+
 /* Private part of the ImageView structure */
-typedef struct {
+struct _ImageViewPrivate {
 	/* Image being displayed */
 	Image *image;
 
@@ -117,7 +121,7 @@ typedef struct {
 
 	/* Whether we need to change the zoom factor */
 	guint need_zoom_change : 1;
-} ImageViewPrivate;
+};
 
 /* Signal IDs */
 enum {
@@ -1241,6 +1245,7 @@ image_view_button_press (GtkWidget *widget, GdkEventButton *event)
 	ImageView *view;
 	ImageViewPrivate *priv;
 	GdkCursor *cursor;
+	int retval;
 
 	view = IMAGE_VIEW (widget);
 	priv = view->priv;
@@ -1253,6 +1258,20 @@ image_view_button_press (GtkWidget *widget, GdkEventButton *event)
 
 	switch (event->button) {
 	case 1:
+		cursor = cursor_get (widget->window, CURSOR_HAND_CLOSED);
+		retval = gdk_pointer_grab (widget->window,
+					   FALSE,
+					   (GDK_POINTER_MOTION_MASK
+					    | GDK_POINTER_MOTION_HINT_MASK
+					    | GDK_BUTTON_RELEASE_MASK),
+					   NULL,
+					   cursor,
+					   event->time);
+		gdk_cursor_destroy (cursor);
+
+		if (retval != 0)
+			return FALSE;
+
 		priv->dragging = TRUE;
 		priv->drag_anchor_x = event->x;
 		priv->drag_anchor_y = event->y;
@@ -1260,16 +1279,6 @@ image_view_button_press (GtkWidget *widget, GdkEventButton *event)
 		priv->drag_ofs_x = priv->xofs;
 		priv->drag_ofs_y = priv->yofs;
 
-		cursor = cursor_get (widget->window, CURSOR_HAND_CLOSED);
-		gdk_pointer_grab (widget->window,
-				  FALSE,
-				  (GDK_POINTER_MOTION_MASK
-				   | GDK_POINTER_MOTION_HINT_MASK
-				   | GDK_BUTTON_RELEASE_MASK),
-				  NULL,
-				  cursor,
-				  event->time);
-		gdk_cursor_destroy (cursor);
 		return TRUE;
 
 	case 4:
@@ -1657,6 +1666,9 @@ image_view_set_zoom (ImageView *view, double zoom)
 
 	priv = view->priv;
 
+	if (zoom > MAX_ZOOM_FACTOR)
+		zoom = MAX_ZOOM_FACTOR;
+
 	if (priv->zoom == zoom)
 		return;
 
